#!/usr/bin/perl
# lycosfwd2mh.pl - A simple perl script to transform a email forwarded by Lycos
# to an approximation of the original message.
# Version 1.1 Feb07
# Copyright 2006 Timothy Pollard
#
#All rights reserved.
#
#Redistribution and use in source and binary forms, with or without modification,
#are permitted provided that the following conditions are met:
#
#* Redistributions of source code must retain the above copyright notice, this 
#list of conditions and the following disclaimer.
#
#* Redistributions in binary form must reproduce the above copyright notice, this
#list of conditions and the following disclaimer in the documentation and/or 
#other materials provided with the distribution.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
#ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
#WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
#DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
#FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
#DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
#SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
#OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
#OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


# Open the file for read.
open (INFILE, "<", $ARGV [0]) or die ("Unable to open the file");

# Read the file into a string. (Handles up to ~100MB files)
read (INFILE, $file, 100000000);

# Close the file.
close (INFILE);

# Generate the new string.
$file = generate_new_string ($file);

# Open the file for write.
open (OUTFILE, ">", $ARGV [0]) or die ("Unable to open the file");

# Write the string to the file.
print OUTFILE $file;

# Close the file.
close (OUTFILE);

# Find the index of the string passed as an argument
sub find_index
{
    $index = $_ [1];
    while ($index <= @file_array.length() / 100)
    {
        if ($file_array [$index] =~ $_ [0])
        {   
            return $index;
        }
        $index++;
    }
    return -1;
}

# Extract the header from the HTML string
# This is a very simple HTML parser which will not support the large majority of
# HTML, but should be sufficient for this purpose.
sub strip_html
{
    $_ = @_ [0];
    s/<\/?\w*>//g;
    s/&lt;/</g;
    s/&gt;/>/g;
    s/&quot;/"/g;
    return $_;
}

# Extracts the headers from the message body
sub extract_headers
{
    $inline_headers_start = find_index ("<br>-----", 0);
    if ($inline_headers_start == -1)
    {   die ("Inline Headers not Found!");  }

    # Form an array by passing the inline header info
    @array = (strip_html ($_ [$inline_headers_start + 1]), 
              strip_html ($_ [$inline_headers_start + 2]), 
              strip_html ($_ [$inline_headers_start + 3]), 
              strip_html ($_ [$inline_headers_start + 4]));

    # Remove the excess characters from the the string
    for ($i = 0; $i < @array; $i++)
    {
        $array [$i] =~ s/^> //;
        $array [$i] =~ s/ :/:/;
    }

    return @array;
}

# Cleans the file, ie removes the content added by Lycos
sub clean
{
    # Delete the header info in the message
    $inline_headers_start = find_index ("<br>------", 0);
    if ($inline_headers_start == -1)
    {   die ("Inline Headers not Found!");  }
    for ($i = $inline_headers_start; $i < $inline_headers_start + 5; $i++)
    {   $_ [$i] = "";   }

    # Find the hr tag
    $hr_index = find_index ("<hr style=\"width:85%;margin-left:0px;text-align:left\"/><br/>", 0); 
    if ($hr_index == -1)
    {   die ("The horizontal rule was not found!");  }

    # Find and delete the add
    $end_add = find_index ("</div>", $hr_index);
    if ($end_add == -1)
    {   die ("End of the Add not Found!");  }

    # Delete the add and the end blockquote tag
    for ($i = $hr_index; $i <= $end_add; $i++)
    {   $_ [$i] = "";   }

    # Delete the >'s
    for ($i = $inline_headers_start; $i < $hr_index; $i++)
    {   $_ [$i] =~ s/^<br>>/<br>/;  }

    return @_;
}

# Performs the actual transformation
sub generate_new_string
{
    # Split on newlines.
    @file_array = split ("\n", $_ [0]);

    # Find the end of the headers.
    $end_header_index = find_index ("This is MIME Preamble", 0);
    if ($end_header_index == -1)
    {
        $end_header_index = find_index ("<br>", 0);
        if ($end_header_index == -1)
        {   die ("End of headers not found.");  }
    }

    # Add a newline to each line
    for ($i = 0; $i < @file_array; $i++)
    {   $file_array [$i] = $file_array [$i] . "\n"; }

    # Get the index of the Content-type header
    $cont_type_header = find_index ("Content-type:", 0);

    # Delete the headers
    for ($i = 0; $i < $end_header_index - 1; $i++)
    {
        if ($i != $cont_type_header)
        {   $file_array [$i] = "";  }
    }

    # Extract the new Headers
    @new_headers = extract_headers (@file_array);
    for ($i = 0; $i < @new_headers; $i++)
    {   $file_array [$i] = $new_headers [$i];   }

    # Clean up the file
    @file_array = clean (@file_array);

    # Reform the string
    $file = $file_array [0];
    for ($i = 1; $i < @file_array; $i++)
    {   $file = $file . $file_array [$i];   }

    return $file;
}
